#!/bin/bash
## Copyright © 2018 Bret Human
## https://cynicaloptimist.me/
##
##    Documentation at:
## https://mage.cynicaloptimist.me/en/smtCtrl
## https://psi.cynicaloptimist.me/Caffarius/smtCtrl
##
## For questions or comments write:
##    info@cynicaloptimist.me
#
# Try your best to control SMT on Ryzen CPUs.
# Usage: ./smt.sh (enable|disable|status)

num_cores=`lscpu | grep "CPU(s):" | grep -v "," | grep -v "-" | cut -d: -f2 | awk '{print $1}'`
num_procs=`nproc`

disable() {
   if [ "${num_procs}" -lt "${num_cores}" ]; then
      echo "SMT has already been disabled!"
   else
      echo -n "Disabling SMT: "
      i="1"
      while [ "${i}" -lt "${num_cores}" ]; do
         echo "0" > /sys/devices/system/cpu/cpu${i}/online
         i=$[ $i + 2 ]
      done
      unset i
      echo "Done."
      return 0
   fi
   return 1
}

enable() {
   if [ "${num_procs}" -eq "${num_cores}" ]; then
      echo "SMT has already been enabled!"
   else
      echo -n "Enabling SMT: "
      i="1"
      while [ "${i}" -lt "${num_cores}" ]; do
         echo "1" > /sys/devices/system/cpu/cpu${i}/online
         i=$[ $i + 2 ]
      done
      unset i
      echo "Done."
      return 0
   fi
   return 1
}

status() {
   cat /proc/cpuinfo |egrep "processor|physical id|core id" | sed 's/^processor/\nprocessor/g'
}

case "$1" in
   disable)
      disable
      ;;
   enable)
      enable
      ;;
   status)
      status
      ;;
   *)
      echo "   Usage: ./smt.sh (disable | enable | status)"
      ;;
esac
